﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Jewels
{
    public partial class GameForm : Form
    {
        Color[] colors = new Color[] { Color.Red, Color.Green, Color.Blue, Color.White, Color.Yellow, Color.Violet};
        static Point noPoint = new Point(-1, -1);

        byte[,] board = new byte[8, 8]; //Sloupec, index
        List<byte>[] refillLists = new List<byte>[8];
        byte[] refillIdxs = new byte[8];
        Point selected = noPoint;
        int Points = 0;

        public GameForm(string startFile, int timeLimit)
        {
            InitializeComponent();

            this.Paint += DrawBackground;

            for (int i = 0; i < 8; i++)
            {
                refillLists[i] = new List<byte>();
            }

            StreamReader sr = new StreamReader(startFile);
            for (int coll = 0; coll < 16; coll++)
            {
                string line = sr.ReadLine();
                char[] chars = line.ToCharArray();
                if (coll < 8)
                {
                    for (int ch = 0; ch < 8; ch++)
                    {
                        board[coll, ch] = byte.Parse(chars[ch].ToString());
                    }
                }
                else
                {
                    foreach (char ch in chars)
                        refillLists[coll - 8].Add(byte.Parse(ch.ToString()));
                }
            }

            if (timeLimit != 0)
            {
                timer1.Interval = timeLimit;
                timer1.Start();
            }
        }

        private void DrawBackground(object sender, PaintEventArgs e)
        {
            //Pozadí pro zásobník
            e.Graphics.FillRectangle(Brushes.LightSkyBlue, 0, 0, 320, 40);

            //Zásobník
            for (int i = 0; i < 8; i++)
            {
                e.Graphics.FillEllipse(new SolidBrush(colors[refillLists[i][refillIdxs[i]] - 1]), i * 40, 0, 38, 38);
            }

            //Pozadí vybaného kamenu
            if (selected != noPoint)
            {
                e.Graphics.FillRectangle(Brushes.CornflowerBlue, selected.X * 40, selected.Y * 40 + 40, 40, 40);
            }

            //Počet bodů
            e.Graphics.DrawString(Points.ToString(), new Font("Arial", 19), Brushes.Black, 0f, 360f);

            //Kameny
            for (int c = 0; c < 8; c++)
            {
                for(int i = 0; i < 8; i++){
                    if (board[c, i] != 255)
                    {
                        Color clr = colors[board[c, i] - 1];
                        e.Graphics.FillEllipse(new SolidBrush(clr), c * 40, i * 40 + 40, 38, 38);
                    }
                }
            }
        }
        /*
        void CheckGameOver()
        {
            for(int x = 0; x < 8 ; x++)
                for (int y = 0; y < 8; y++)
                {
                    if(x>0)
                        if()
                }
        }
        */
        private void GameForm_MouseClick(object sender, MouseEventArgs e)
        {
            if ((e.X < 320) && (e.Y < 360) && (e.Y >=40)) //Myš je v hrací oblasti
            {
                if (selected == noPoint)
                {
                    selected = new Point(e.X / 40, (e.Y -40) / 40);
                    this.Invalidate();
                }
                else
                {
                    Point p2 = new Point(e.X / 40, (e.Y -40)/ 40);
                    byte p1 = board[selected.X, selected.Y];
                    SwitchColors(selected, p2);
                    //Vyhodnotí sledy, nalezené sledy ihned odebere
                    int pointsReceived = EvaluateBoard();
                    //Pokud vznikne alespoň jeden sled
                    if (pointsReceived > 0)
                    {
                        //Nechá barvy jak jsou a aktualizuje
                        RefillBoard(); //Naplní sloupce, kde byly odebrány sledy
                        while(EvaluateBoard() > 0) //Dokud bude na ploše nějaký sled 
                            RefillBoard();
                        Deselect();
                    }
                    else
                    {
                        //Vrátí barvy zpět
                        SwitchColors(selected, p2);
                        Deselect();
                    }
                }
            }
            else Deselect();
        }

        private void RefillBoard()
        {
            for (int coll = 0; coll < 8; coll++)
                for (int i = 7; i >= 0; i--)
                {
                    if (board[coll, i] == 255)
                    {
                        for (int tI = i - 1; tI >= -1; tI--)
                        {
                            if (tI == -1)
                            {
                                board[coll, i] = refillLists[coll][refillIdxs[coll]];
                                refillIdxs[coll]++;
                                if (refillIdxs[coll] == refillLists[coll].Count) refillIdxs[coll] = 0;
                                Test();
                                break;
                            }
                            else if (board[coll, tI] != 255)
                            {
                                board[coll, i] = board[coll, tI];
                                board[coll, tI] = 255;
                                Test();
                                break;
                            }
                        }
                    }
                }
        }

        void Test()
        {
            this.Refresh();
            System.Threading.Thread.Sleep(100);
        }

        //Prohledá hrací plochu na sledy a vrátí počet bodů
        private int EvaluateBoard()
        {
            int[,] points = new int[8, 8];
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    CheckField(new Point(x, y), ref points);
                }
            }

            int totalPoints = 0;
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    totalPoints += points[x, y];
                }
            }

            Points += totalPoints;
            return totalPoints;
        }

        private void CheckField(Point p, ref int[,] points)
        {
            byte color = board[p.X, p.Y];

            if((p.X >= 1) && (p.X <=6))
                if ((board[p.X - 1, p.Y] == color) && (board[p.X + 1, p.Y] == color))
                {
                    board[p.X, p.Y] = 255;
                    board[p.X - 1, p.Y] = 255;
                    board[p.X + 1, p.Y] = 255;

                    points[p.X, p.Y]++;
                    points[p.X - 1, p.Y]++;
                    points[p.X + 1, p.Y]++;
                }
            if ((p.Y >= 1) && (p.Y <= 6))
                if ((board[p.X, p.Y - 1] == color) && (board[p.X, p.Y + 1] == color))
                {
                    board[p.X, p.Y] = 255;
                    board[p.X, p.Y - 1] = 255;
                    board[p.X, p.Y + 1] = 255;

                    points[p.X, p.Y]++;
                    points[p.X, p.Y - 1]++;
                    points[p.X, p.Y + 1]++;
                }
        }

        private void SwitchColors(Point p1, Point p2)
        {
            byte val1 = board[p1.X, p1.Y];
            board[p1.X, p1.Y] = board[p2.X, p2.Y];
            board[p2.X, p2.Y] = val1;
        }

        private void Deselect()
        {
            selected = noPoint;
            this.Invalidate();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            MouseClick -= GameForm_MouseClick; //Zabrání dalšímu hraní
            timer1.Stop();
            MessageBox.Show("Uplynul časový limit", "Konec hry");
        }
    }
}
